from .component import Component
from . import exceptions
from .peapy import PeaPy


class CircleCollider(Component):
    def __init__(self, x_offset=0, y_offset=0, radius=None):
        super().__init__()

        self.x_offset = x_offset
        self.y_offset = y_offset
        self.radius = radius

    # Called when the component is created
    def init(self, game: PeaPy, obj_name: str) -> PeaPy:
        self.peapy = game
        self.obj_name = obj_name

        # Init component

        return self.peapy

    # Called every frame
    def update(self, game: PeaPy, obj_name: str) -> PeaPy:
        self.peapy = game  # The parent PeaPy object
        self.obj_name = obj_name  # The name of the parent object

        # Update component

        return self.peapy

    def is_colliding(self, target: str) -> bool:
        """
        Check if this Collider is colliding with another Collider
        """
        if "CircleCollider" in self.peapy[target].get_components():
            d_x = abs(
                self.peapy[self.obj_name]["Transform"].x
                + self.x_offset
                - self.peapy[target]["Transform"].x
                - self.peapy[target]["CircleCollider"].x_offset
            )
            d_y = abs(
                self.peapy[self.obj_name]["Transform"].y
                + self.y_offset
                - self.peapy[target]["Transform"].y
                - self.peapy[target]["CircleCollider"].y_offset
            )
            d = (d_x**2 + d_y**2) ** 0.5

            try:
                if self.radius is None:
                    r1 = self.peapy[self.obj_name]["Transform"].height
                else:
                    r1 = self.radius
                if self.peapy[target]["CircleCollider"].radius is None:
                    r2 = self.peapy[target]["Transform"].height
                else:
                    r2 = self.peapy[target]["CircleCollider"].radius
            except exceptions.ComponentNotFoundException:
                raise exceptions.RequiredComponentNotPresent("Transform is required")

            return d <= r1 + r2

        elif "RectCollider" in self.peapy[target].get_components():
            radius = (
                self.peapy[self.obj_name]["Transform"].height
                if self.radius is None
                else self.radius
            )
            x = self.peapy[self.obj_name]["Transform"].x + self.x_offset
            y = self.peapy[self.obj_name]["Transform"].y + self.y_offset

            rx1 = (
                self.peapy[target]["Transform"].x
                + self.peapy[target]["RectCollider"].x_offset
            )
            ry1 = (
                self.peapy[target]["Transform"].y
                + self.peapy[target]["RectCollider"].y_offset
            )

            if self.peapy[target]["RectCollider"].width is None:
                rx2 = rx1 + self.peapy[target]["Transform"].width
            else:
                rx2 = rx1 + self.peapy[target]["RectCollider"].width

            if self.peapy[target]["RectCollider"].height is None:
                ry2 = ry1 + self.peapy[target]["Transform"].height
            else:
                ry2 = ry1 + self.peapy[target]["RectCollider"].height

            x_nearest = max(rx1, min(x, rx2))
            y_nearest = max(ry1, min(y, ry2))

            x_distance = x_nearest - x
            y_distance = y_nearest - y

            return (x_distance**2 + y_distance**2) < (radius**2)

        else:
            raise exceptions.RequiredComponentNotPresent(
                "Target needs to have a Collider"
            )
