from .component import Component
from . import exceptions
from .peapy import PeaPy
from shapely.geometry import Polygon, Point


class RectCollider(Component):
    """
    PeaPy RectCollider component
    """

    def __init__(self, x_offset=0, y_offset=0, width=None, height=None):
        """
        Construct a new RectCollider component

        Args:
            x_offset (int): The x offset to the transform of the parent object
            y_offset (int): The y offset to the transform of the parent object
            width (int): The width of the collider. When None, the width of the parent object's Transform is used
            height (int): The height of the collider. When None, the height of the parent object's Transform is used
        """
        super().__init__()

        self.x_offset = x_offset
        self.y_offset = y_offset
        self.width = width
        self.height = height

    def init(self, game: PeaPy, obj_name: str) -> PeaPy:
        self.peapy = game
        self.obj_name = obj_name

        # Init object

        return self.peapy

    # Called every frame
    def update(self, game: PeaPy, obj_name: str) -> PeaPy:
        self.peapy = game  # The parent PeaPy object
        self.obj_name = obj_name  # The name of the parent object

        # Update component

        return self.peapy

    def is_colliding(self, target: str) -> bool:
        """
        Check if this Collider is colliding with another Collider
        """
        if "RectCollider" in self.peapy[target].get_components():
            p1 = Polygon(
                [
                    self.peapy[self.obj_name]["Transform"].top_left,
                    self.peapy[self.obj_name]["Transform"].top_right,
                    self.peapy[self.obj_name]["Transform"].bottom_right,
                    self.peapy[self.obj_name]["Transform"].bottom_left,
                ]
            )

            p2 = Polygon(
                [
                    self.peapy[target]["Transform"].top_left,
                    self.peapy[target]["Transform"].top_right,
                    self.peapy[target]["Transform"].bottom_right,
                    self.peapy[target]["Transform"].bottom_left,
                ]
            )

        elif "CircleCollider" in self.peapy[target].get_components():
            radius = self.peapy[target]["Transform"].height
            x = self.peapy[target]["Transform"].x + self.x_offset
            y = self.peapy[target]["Transform"].y + self.y_offset

            rx1 = (
                self.peapy[self.obj_name]["Transform"].x
                + self.peapy[self.obj_name]["RectCollider"].x_offset
            )
            ry1 = (
                self.peapy[self.obj_name]["Transform"].y
                + self.peapy[self.obj_name]["RectCollider"].y_offset
            )

            if self.peapy[self.obj_name]["RectCollider"].width is None:
                rx2 = rx1 + self.peapy[self.obj_name]["Transform"].width
            else:
                rx2 = rx1 + self.peapy[self.obj_name]["RectCollider"].width

            if self.peapy[self.obj_name]["RectCollider"].height is None:
                ry2 = ry1 + self.peapy[self.obj_name]["Transform"].height
            else:
                ry2 = ry1 + self.peapy[self.obj_name]["RectCollider"].height

            x_nearest = max(rx1, min(x, rx2))
            y_nearest = max(ry1, min(y, ry2))

            x_distance = x_nearest - x
            y_distance = y_nearest - y

            return (x_distance**2 + y_distance**2) < (radius**2)

        else:
            raise exceptions.RequiredComponentNotPresent(
                "Target needs to have a Collider"
            )

        return p1.intersects(p2)

    def __repr__(self):
        return f"components.{self.__class__.__name__}()"
